#!/usr/bin/env ruby
#
# Trust boundary analysis library
#
# rights.rb: Describes access rights for each known object type 
#
# mmiller@hick.org
# 4/2008
#

module TBA

class Medium

	# The rights associated with individual object types
	Rights =
		{
			"Adapter" => { },

			"ALPC Port" => 
				{
					"GR" => 0x020000,
					"GW" => 0x020000,
					"GX" => 0x120001,
					"GA" => 0x1f0001,

					"Connect" => 0x0001
				},

			"Callback" => { },
			"Controller" => { },

			"Desktop" =>
				{
					"GR" => 0x120041,
					"GW" => 0x1200be,
					"GX" => 0x120100,
					"GA" => 0x1f01ff,

					"ReadObjects"     => 0x0001,
					"CreateWindow"    => 0x0002,
					"CreateMenu"      => 0x0004,
					"HookControl"     => 0x0008,
					"JournalRecord"   => 0x0010,
					"JournalPlayback" => 0x0020,
					"Enumerate"       => 0x0040,
					"WriteObjects"    => 0x0080,
					"SwitchDesktop"   => 0x0100,
				},

			"Device" => { },
			"Directory" => { },
			"Driver" => { },
			"EtwRegistration" => { },

			"Event" =>
				{
					"GR" => 0x120001,
					"GW" => 0x120002,
					"GX" => 0x120000,
					"GA" => 0x1f0003,

					"QueryState"  => 0x1,
					"ModifyState" => 0x2
				},

			"File" =>
				{
					"GR" => 0x120089,
					"GW" => 0x120116,
					"GX" => 0x1200a0,
					"GA" => 0x1f01ff,
		
					"ListDirectory"   => 0x0001,
					"ReadData"        => 0x0001,
					"AddFile"         => 0x0002,
					"WriteData"       => 0x0002,
					"CreatePipe"      => 0x0004,
					"AppendData"      => 0x0004,
					"AddSubDirectory" => 0x0004,
					"ReadEa"          => 0x0008,
					"WriteEa"         => 0x0010,
					"Execute"         => 0x0020,
					"Traverse"        => 0x0020,
					"DeleteChild"     => 0x0040,
					"ReadAttributes"  => 0x0080,
					"WriteAttributes" => 0x0100,
				},

			"IoCompletion" => { },

			"Job" => 
				{
					"GR" => 0x120004,
					"GW" => 0x12000b,
					"GX" => 0x120000,
					"GA" => 0x1f001f,
		
					"AssignProcess"         => 0x0001,
					"SetAttributes"         => 0x0002,
					"Query"                 => 0x0004,
					"Terminate"             => 0x0008,
					"SetSecurityAttributes" => 0x0010,
				},

			"Key" => 
				{
					"GR" => 0x20019,
					"GW" => 0x20006,
					"GX" => 0x20039,
					"GA" => 0xf003f,
		
					"QueryValue"   => 0x0001,
					"SetValue"     => 0x0002,
					"CreateSubKey" => 0x0004,
					"EnumSubKeys"  => 0x0008,
					"Notify"       => 0x0010,
					"CreateLink"   => 0x0020,
					"Wow6464Key"   => 0x0100,
					"Wow6432Key"   => 0x0200,
				},

			"KeyedEvent" => { },

			"Mutant" =>
				{
					"GR" => 0x120000,
					"GW" => 0x120001,
					"GX" => 0x120000,
					"GA" => 0x1f0001,

					"ModifyState" => 0x1
				},

			"Process" =>
				{
					"GR" => 0x020410,
					"GW" => 0x020bea,
					"GX" => 0x121001,
					"GA" => 0x1fffff,

					"Terminate"               => 0x0001,
					"CreateThread"            => 0x0002,
					"VmOperation"             => 0x0008,
					"VmRead"                  => 0x0010,
					"VmWrite"                 => 0x0020,
					"DupHandle"               => 0x0040,
					"CreateProcess"           => 0x0080,
					"SetQuota"                => 0x0100,
					"SetInformation"          => 0x0200,
					"QueryInformation"        => 0x1400,
					"SuspendResume"           => 0x0800,
					"QueryLimitedInformation" => 0x1000,
				},

			"Section" =>
				{
					"GR" => 0x20005,
					"GW" => 0x20002,
					"GX" => 0x20008,
					"GA" => 0xf001f,

					"Query"              => 0x0001,
					"MapWrite"           => 0x0002,
					"MapRead"            => 0x0004,
					"MapExecute"         => 0x0008,
					"ExtendSize"         => 0x0010,
					"MapExecuteExplicit" => 0x0020,
				},

			"Semaphore" =>
				{
					"GR" => 0x120001,
					"GW" => 0x120002,
					"GX" => 0x120000,
					"GA" => 0x1f0003,

					"QueryState"  => 0x1,
					"ModifyState" => 0x2
				},

			"Service" => 
				{
					"GR" => 0x2008d,
					"GW" => 0x20002,
					"GX" => 0x20170,
					"GA" => 0xf01ff,

					"QueryConfig"    => 0x0001,
					"ChangeConfig"   => 0x0002,
					"QueryStatus"    => 0x0004,
					"EnumDependents" => 0x0008,
					"Start"          => 0x0010,
					"Stop"           => 0x0020,
					"Pause/Continue" => 0x0040,
					"Interrogate"    => 0x0080,
					"User Defined"   => 0x0100
				},

			"Session" =>
				{
					"GR" => 0x20001,
					"GW" => 0x20002,
					"GX" => 0x20000,
					"GA" => 0xf0003,

					"Query"  => 0x0001,
					"Modify" => 0x0002
				},
		
			"SymbolicLink" => { },

			"Thread" =>
				{
					"GR" => 0x020048,
					"GW" => 0x020437,
					"GX" => 0x120800,
					"GA" => 0x1fffff,

					"Terminate"               => 0x0001,
					"Suspend/Resume"          => 0x0002,
					"GetContext"              => 0x0008,
					"SetContext"              => 0x0010,
					"SetInformation"          => 0x0020,
					"QueryInformation"        => 0x0040,
					"SetThreadToken"          => 0x0080,
					"Impersonate"             => 0x0100,
					"DirectImpersonation"     => 0x0200,
					"SetLimitedInformation"   => 0x0400,
					"QueryLimitedInformation" => 0x0800
				},

			"Timer" =>
				{
					"GR" => 0x020001,
					"GW" => 0x020002,
					"GX" => 0x120000,
					"GA" => 0x1f0003,

					"QueryState"  => 0x1,
					"ModifyState" => 0x2
				},

			"TmEn" => { },
			"TmRm" => { }, 
			"TmTm" => { },
			"TmTx" => { },

			"Token" =>
				{
					"GR" => 0x020008,
					"GW" => 0x0200e0,
					"GX" => 0x120000,
					"GA" => 0x1f01ff,

					"AssignPrimary"    => 0x0001,
					"Duplicate"        => 0x0002,
					"Impersonate"      => 0x0004,
					"Query"            => 0x0008,
					"QuerySource"      => 0x0010,
					"AdjustPrivileges" => 0x0020,
					"AdjustGroups"     => 0x0040,
					"AdjustDefault"    => 0x0080,
					"AdjustSessionId"  => 0x0100
				},

			"TpWorkerFactory" => { },

			"WindowStation" =>
				{
					"GR" => 0x020303,
					"GW" => 0x02001c,
					"GX" => 0x120060,
					"GA" => 0x1f01ff,

					"EnumDesktops"      => 0x0001,
					"ReadAttributes"    => 0x0002,
					"AccessClipboard"   => 0x0004,
					"CreateDesktop"     => 0x0008,
					"WriteAttributes"   => 0x0010,
					"AccessGlobalAtoms" => 0x0020,
					"ExitWindows"       => 0x0040,
					"Enumerate"         => 0x0100,
					"ReadScreen"        => 0x0200,
				},

			"WmiGuid" => { },
		}

end

end
