using System;
using System.Security.Principal;
using System.Security.AccessControl;

namespace TBA.Database
{
    partial class TBADataContext
    {
        /// <summary>
        /// The snapshot that queries should be run against
        /// </summary>
        public int SnapshotId { get; set; }
    }

    public interface IDataFlowHalf
    {
        ExecutionContext ExecutionContext { get; set; }
        Actor Actor { get; set; }
        Medium Medium { get; set; }
        Verb Verb { get; set; }
        CallStack CallStack { get; set; }
    }

    public partial class Verb
    {
        public override int GetHashCode()
        {
            return Id;
        }

        public override bool Equals(object obj)
        {
            if (obj is Verb)
                return (obj as Verb).Id == Id;
            else
                return false;
        }

        public override string ToString()
        {
            return Name;
        }
    }

    public partial class Actor
    {
        /// <summary>
        /// The security identifier class for this actor
        /// </summary>
        public SecurityIdentifier Sid
        {
            get
            {
                // TODO: add a check on ActorType when needed in the future

                return new SecurityIdentifier(Name);
            }
        }

        /// <summary>
        /// Converts the SID to an NT account name
        /// </summary>
        /// <returns>The actor as an NT account name</returns>
        public override string ToString()
        {
            return Sid.GetNtAccountName();
        }
    }

    public partial class Medium
    {
        public override string ToString()
        {
            return Name;
        }

        public bool IsCached
        {
            get
            {
                if (!isCached)
                {
                    isCached = true;
                    return false;
                }
                else
                    return true;
            }
        }
        private bool isCached = false;
    }

    public partial class DataFlowDefinition
        : IDataFlowHalf
    {
        public override int GetHashCode()
        {
            return String.Format("{0}-{1}-{2}",
                ActorId,
                MediumId,
                VerbId).GetHashCode();
        }

        public override string ToString()
        {
            return String.Format(
                "<{0},{1},{2}>",
                Actor,
                Medium,
                Verb);
        }
    }

    public partial class DataFlowUse
        : IDataFlowHalf
    {
        public override int GetHashCode()
        {
            return String.Format("{0}-{1}-{2}",
                ActorId,
                MediumId,
                VerbId).GetHashCode();
        }

        public override string ToString()
        {
            return String.Format(
                "<{0},{1},{2}>",
                Actor,
                Medium,
                Verb);
        }
    }

    public partial class DataFlow
    {
        public override string ToString()
        {
            return String.Format(
                "{0} --> {1}",
                DataFlowDefinition,
                DataFlowUse);
        }
    }

    public static class SidExtension
    {
        public static string GetNtAccountName(this SecurityIdentifier sid)
        {
            try
            {
                NTAccount account = (NTAccount)sid.Translate(typeof(NTAccount));
                return account.Value;
            }
            catch (IdentityNotMappedException)
            {
                return sid.Value;
            }
        }
    }
}
