//
// Dynamic trust boundary metadata collection driver
//
// mmiller@hick.org 
// 3/2008
//
#ifndef _DYNBA_LOG_H
#define _DYNBA_LOG_H

//
// An enumeration of tasks that can be performed
//
typedef enum _OBJECT_LOG_TASK
{
	ObjectLogTaskInit,
	ObjectLogTaskWrite,
	ObjectLogTaskQueryName
} OBJECT_LOG_TASK, *POBJECT_LOG_TASK;

//
// This structure contains information needed to instrument the creation of
// objects of a given object type.
//
typedef struct _OBJECT_LOG
{
	//
	// The unique key representing this object type
	//
	ULONG_PTR Key;

	//
	// A pointer to the object type being instrumented
	//
	POBJECT_TYPE ObjectType;

	//
	// A handle to the instrumentation log file for this object type
	//
	HANDLE FileHandle;

	//
	// The actual SecurityProcedure for the object type represented by this
	// object log.  This is cached here because we hook the active one with our
	// own implementation.
	//
	POBJECT_TYPE_SECURITY_PROCEDURE SecurityProcedure;

	//
	// The handle we get back from calling ObRegisterCallbacks
	//
	PVOID ObRegistrationHandle;

} OBJECT_LOG, *POBJECT_LOG;

//
// A work item that is used to log information to an object log
//
typedef struct _OBJECT_LOG_WORKITEM
{
	//
	// List entry used to track the work item if it is queued before the object
	// log directory has been opened
	//
	LIST_ENTRY QueueEntry;

	//
	// The task to perform
	//
	OBJECT_LOG_TASK Task;

	union
	{
		//
		// Parameters needed to complete the initialization of an object log
		//
		struct
		{
			POBJECT_LOG ObjectLog;
		} Init;

		//
		// Associated with the Write task.  Object can be NULL if ObjectLog is
		// non-NULL.
		//
		struct
		{ 
			PVOID Object;
			POBJECT_LOG ObjectLog;

			POBJECT_ACCESS_METADATA AccessMetadata;
			POBJECT_SECURITY_CONTEXT_METADATA SecurityContextMetadata;
			POBJECT_SECURITY_DESCRIPTOR_METADATA SecurityDescriptorMetadata;
			POBJECT_NAME_METADATA NameMetadata;
			POBJECT_CALLSTACK_METADATA CallStackMetadata;
			POBJECT_IMAGEINFO_METADATA ImageInfoMetadata;
		} Write;

		//
		// Queries the name of an object and logs it to the appropriate location
		//
		struct
		{
			PVOID Object;
			ULONG Count;
		} QueryName;

	};

} OBJECT_LOG_WORKITEM, *POBJECT_LOG_WORKITEM;

NTSTATUS 
LookupObjectLog(
	__in PVOID Object,
	__out POBJECT_LOG *OutObjectLog);

POBJECT_LOG_WORKITEM 
AllocateObjectLogWorkItem(
	__in OBJECT_LOG_TASK Task);

VOID
QueueObjectLogWorkItem(
	__in POBJECT_LOG_WORKITEM WorkItem);

VOID
FreeObjectLogWorkItem(
	__in POBJECT_LOG_WORKITEM WorkItem);

ULONG64
GetNextObjectLogSequence();

NTSTATUS
InitializeObjectLog();

extern 
PETHREAD ObjectLogWorkerThread;

#endif
