//
// Dynamic trust boundary metadata collection driver
//
// mmiller@hick.org 
// 3/2008
//
#ifndef DYNBA_NTPROTO_H
#define DYNBA_NTPROTO_H

typedef enum _SECURITY_OPERATION
{
	OpSetSecurityDescriptor,
	OpQuerySecurityDescriptor,
	OpDeleteSecurityDescriptor,
	OpAssignSecurityDescriptor
} SECURITY_OPERATION, *PSECURITY_OPERATION;

typedef enum _OPEN_OPERATION
{
	OpenOpCreateHandle,
	OpenOpOpenHandle,
	OpenOpDuplicateHandle,
	OpenOpInheritHandle
} OPEN_OPERATION, *POPEN_OPERATION;

typedef NTSTATUS (*POBJECT_TYPE_SECURITY_PROCEDURE)(
	__in PVOID Object,
	__in SECURITY_OPERATION Operation,
	__in PSECURITY_INFORMATION Information,
	__in PSECURITY_DESCRIPTOR SecurityDescriptor,
	__inout PULONG CapturedLength,
	__inout PSECURITY_DESCRIPTOR *ObjectSecurityDescriptor,
	__in POOL_TYPE PoolType,
	__in PGENERIC_MAPPING GenericMapping);

typedef NTSTATUS (*POBJECT_TYPE_OPEN_PROCEDURE)(
	__in OPEN_OPERATION Operation,
	__in ULONG HandleCount,
	__in PEPROCESS ProcessObject,
	__in PVOID Object,
	__in PACCESS_MASK GrantedAccess);

typedef struct _OBJECT_TYPE_INITIALIZER_VISTASP0
{
	USHORT Length;
	UCHAR ObjectTypeFlags;
	ULONG ObjectTypeCode;
	ULONG InvalidAttributes;
	GENERIC_MAPPING GenericMapping;
	ULONG ValidAccessMask;
	POOL_TYPE PoolType;
	ULONG DefaultPagedPoolCharge;
	ULONG DefaultNonPagedPoolCharge;
	PVOID DumpProcedure;
	PVOID OpenProcedure;
	PVOID CloseProcedure;
	PVOID DeleteProcedure;
	PVOID ParseProcedure;
	POBJECT_TYPE_SECURITY_PROCEDURE SecurityProcedure;
	PVOID QueryNameProcedure;
	PVOID OkayToCloseProcedure;
} OBJECT_TYPE_INITIALIZER_VISTASP0, *POBJECT_TYPE_INITIALIZER_VISTASP0;

typedef struct _OBJECT_TYPE_VISTASP0
{
	ERESOURCE Mutex;
	LIST_ENTRY TypeList;
	UNICODE_STRING Name;
	PVOID DefaultObject;
	ULONG Index;
	ULONG TotalNumberOfObjects;
	ULONG TotalNumberOfHandles;
	ULONG HighWaterNumberOfObjects;
	ULONG HighWaterNumberOfHandles;
	OBJECT_TYPE_INITIALIZER_VISTASP0 TypeInfo;
} OBJECT_TYPE_VISTASP0, *POBJECT_TYPE_VISTASP0;

/*
 * kd> dt nt!_OBJECT_TYPE_INITIALIZER
 *  +0x000 Length           : Uint2B
 *  +0x002 ObjectTypeFlags  : UChar
 *  +0x002 CaseInsensitive  : Pos 0, 1 Bit
 *  +0x002 UnnamedObjectsOnly : Pos 1, 1 Bit
 *  +0x002 UseDefaultObject : Pos 2, 1 Bit
 *  +0x002 SecurityRequired : Pos 3, 1 Bit
 *  +0x002 MaintainHandleCount : Pos 4, 1 Bit
 *  +0x002 MaintainTypeList : Pos 5, 1 Bit
 *  +0x002 SupportsObjectCallbacks : Pos 6, 1 Bit
 *  +0x004 ObjectTypeCode   : Uint4B
 *  +0x008 InvalidAttributes : Uint4B
 *  +0x00c GenericMapping   : _GENERIC_MAPPING
 *  +0x01c ValidAccessMask  : Uint4B
 *  +0x020 RetainAccess     : Uint4B
 *  +0x024 PoolType         : _POOL_TYPE
 *  +0x028 DefaultPagedPoolCharge : Uint4B
 *  +0x02c DefaultNonPagedPoolCharge : Uint4B
 *  +0x030 DumpProcedure    : Ptr64     void 
 *  +0x038 OpenProcedure    : Ptr64     long 
 *  +0x040 CloseProcedure   : Ptr64     void 
 *  +0x048 DeleteProcedure  : Ptr64     void 
 *  +0x050 ParseProcedure   : Ptr64     long 
 *  +0x058 SecurityProcedure : Ptr64     long 
 *  +0x060 QueryNameProcedure : Ptr64     long 
 *  +0x068 OkayToCloseProcedure : Ptr64     unsigned char 
 */

typedef struct _OBJECT_TYPE_INITIALIZER
{
	USHORT Length;
	union
	{
		UCHAR ObjectTypeFlags;
		struct
		{
			USHORT CaseInsensitive         : 1;
			USHORT UnnamedObjectsOnly      : 1;
			USHORT UseDefaultObject        : 1;
			USHORT SecurityRequired        : 1;
			USHORT MaintainHandleCount     : 1;
			USHORT MaintainTypeList        : 1;
			USHORT SupportsObjectCallbacks : 1;
		};
	};
	ULONG ObjectTypeCode;
	ULONG InvalidAttributes;
	GENERIC_MAPPING GenericMapping;
	ULONG ValidAccessMask;
	ULONG RetainAccess;
	POOL_TYPE PoolType;
	ULONG DefaultPagedPoolCharge;
	ULONG DefaultNonPagedPoolCharge;
	PVOID DumpProcedure;
	PVOID OpenProcedure;
	PVOID CloseProcedure;
	PVOID DeleteProcedure;
	PVOID ParseProcedure;
	POBJECT_TYPE_SECURITY_PROCEDURE SecurityProcedure;
	PVOID QueryNameProcedure;
	PVOID OkayToCloseProcedure;
} OBJECT_TYPE_INITIALIZER, *POBJECT_TYPE_INITIALIZER;

/*
 * kd> dt nt!_OBJECT_TYPE
 *  +0x000 TypeList         : _LIST_ENTRY
 *  +0x010 Name             : _UNICODE_STRING
 *  +0x020 DefaultObject    : Ptr64 Void
 *  +0x028 Index            : Uint4B
 *  +0x02c TotalNumberOfObjects : Uint4B
 *  +0x030 TotalNumberOfHandles : Uint4B
 *  +0x034 HighWaterNumberOfObjects : Uint4B
 *  +0x038 HighWaterNumberOfHandles : Uint4B
 *  +0x040 TypeInfo         : _OBJECT_TYPE_INITIALIZER
 *  +0x0b0 Mutex            : _ERESOURCE
 *  +0x118 TypeLock         : _EX_PUSH_LOCK
 *  +0x120 Key              : Uint4B
 *  +0x128 ObjectLocks      : [32] _EX_PUSH_LOCK
 *  +0x228 CallbackList     : _LIST_ENTRY
 */

typedef struct _OBJECT_TYPE
{
	LIST_ENTRY TypeList;
	UNICODE_STRING Name;
	PVOID DefaultObject;
	ULONG Index;
	ULONG TotalNumberOfObjects;
	ULONG TotalNumberOfHandles;
	ULONG HighWaterNumberOfObjects;
	ULONG HighWaterNumberOfHandles;
	OBJECT_TYPE_INITIALIZER TypeInfo;
	ERESOURCE Mutex;
	ULONG64 TypeLock;
	ULONG Key;
	ULONG64 ObjectLocks;
	LIST_ENTRY CallbackList;
} OBJECT_TYPE, *POBJECT_TYPE;

#define GET_OBJECT_HEADER(Object) ((POBJECT_HEADER)((PUCHAR)Object - sizeof(OBJECT_HEADER)))

typedef struct _OBJECT_HEADER
{
	UINT32 PointerCount;
	union
	{
		UINT32 HandleCount;
		PVOID NextToFree;
	};
	POBJECT_TYPE Type;
	UCHAR NameInfoOffset;
	UCHAR HandleInfoOffset;
	UCHAR QuotaInfoOffset;
	UCHAR Flags;
	PVOID ObjectCreateInfoOrQuotaBlockCharged;
	PSECURITY_DESCRIPTOR SecurityDescriptor;
} OBJECT_HEADER, *POBJECT_HEADER;

typedef struct _LDR_DATA_TABLE_ENTRY
{
	LIST_ENTRY InLoadOrderLinks;
	LIST_ENTRY InMemoryOrderLinks;
	LIST_ENTRY InInitializationOrderLinks;
	ULONG_PTR DllBase;
	ULONG_PTR EntryPoint;
	ULONG SizeOfImage;
	UNICODE_STRING FullDllName;
	UNICODE_STRING BaseDllName;
} LDR_DATA_TABLE_ENTRY, *PLDR_DATA_TABLE_ENTRY;

typedef struct _PEB_LDR_DATA
{
	ULONG Length;
	ULONG Initialized;
	PVOID SsHandle;
	LIST_ENTRY InLoadOrderModuleList;
	LIST_ENTRY InMemoryOrderModuleList;
	LIST_ENTRY InInitializedOrderModuleList;
	PVOID EntryInProgress;
} _PEB_LDR_DATA, *PPEB_LDR_DATA;

typedef struct _PEB
{
	UCHAR InheritedAddressSpace;
	UCHAR ReadImageFileExecOptions;
	UCHAR BeingDebugged;
	UCHAR BitField;
	UCHAR ImageUsesLargePages : 1;
	UCHAR IsProtectedProcess : 1;
	UCHAR IsLegacyProcess : 1;
	UCHAR IsImageDynamicallyRelocated : 1;
	UCHAR SpareBits : 4;
	PVOID Mutant;
	PVOID ImageBaseAddress;
	PPEB_LDR_DATA Ldr;
} PEB, *PPEB;

extern NTSYSAPI NTSTATUS NTAPI ObQueryNameString(
	__in PVOID Object,
	__in POBJECT_NAME_INFORMATION Information,
	__in ULONG MaximumLength,
	__out PULONG ActualLength);
extern NTSYSAPI NTSTATUS NTAPI ZwQueryInformationProcess(
	__in HANDLE ProcessHandle,
	__in PROCESSINFOCLASS ProcessInformationClass,
	__out PVOID ProcessInformation,
	__in ULONG ProcessInformationLength,
	__out PULONG ReturnLength OPTIONAL);
extern NTSYSAPI PIMAGE_NT_HEADERS NTAPI RtlImageNtHeader(
	__in PVOID Base);
extern PVOID RtlImageDirectoryEntryToData(
	__in PVOID ImageBase,
	__in BOOLEAN UnknownBoolean,
	__in ULONG ImageDirectoryEntry,
	__out PULONG SectionSize);
extern PVOID RtlPcToFileHeader(
	__in PVOID Symbol,
	__out PVOID *ImageBase);

#define SE_OWNER_DEFAULTED               (0x0001)
#define SE_GROUP_DEFAULTED               (0x0002)
#define SE_DACL_PRESENT                  (0x0004)
#define SE_DACL_DEFAULTED                (0x0008)
#define SE_SACL_PRESENT                  (0x0010)
#define SE_SACL_DEFAULTED                (0x0020)
#define SE_DACL_AUTO_INHERIT_REQ         (0x0100)
#define SE_SACL_AUTO_INHERIT_REQ         (0x0200)
#define SE_DACL_AUTO_INHERITED           (0x0400)
#define SE_SACL_AUTO_INHERITED           (0x0800)
#define SE_DACL_PROTECTED                (0x1000)
#define SE_SACL_PROTECTED                (0x2000)
#define SE_RM_CONTROL_VALID              (0x4000)
#define SE_SELF_RELATIVE                 (0x8000)

#endif
